@@ -115,7 +115,8 @@ class WorkOnDiskTestCase(base.IronicLibTestCase):
                                              self.node_uuid, commit=True,
                                              boot_option="netboot",
                                              boot_mode="bios",
-                                             disk_label=None)
+                                             disk_label=None,
+                                             cpu_arch="")
 
     def test_no_swap_partition(self):
         self.mock_ibd.side_effect = iter([True, False])
@@ -133,7 +134,8 @@ class WorkOnDiskTestCase(base.IronicLibTestCase):
                                              self.node_uuid, commit=True,
                                              boot_option="netboot",
                                              boot_mode="bios",
-                                             disk_label=None)
+                                             disk_label=None,
+                                             cpu_arch="")
 
     def test_no_ephemeral_partition(self):
         ephemeral_part = '/dev/fake-part1'
@@ -160,7 +162,8 @@ class WorkOnDiskTestCase(base.IronicLibTestCase):
                                              self.node_uuid, commit=True,
                                              boot_option="netboot",
                                              boot_mode="bios",
-                                             disk_label=None)
+                                             disk_label=None,
+                                             cpu_arch="")
 
     @mock.patch.object(utils, 'unlink_without_raise', autospec=True)
     @mock.patch.object(disk_utils, '_get_configdrive', autospec=True)
@@ -193,12 +196,48 @@ class WorkOnDiskTestCase(base.IronicLibTestCase):
                                              commit=True,
                                              boot_option="netboot",
                                              boot_mode="bios",
-                                             disk_label=None)
+                                             disk_label=None,
+                                             cpu_arch="")
         mock_unlink.assert_called_once_with('fake-path')
 
     @mock.patch.object(utils, 'mkfs', lambda fs, path, label=None: None)
     @mock.patch.object(disk_utils, 'block_uuid', lambda p: 'uuid')
-    @mock.patch.object(disk_utils, 'populate_image', lambda *_: None)
+    @mock.patch.object(disk_utils, 'populate_image', autospec=True)
+    def test_without_image(self, mock_populate):
+        ephemeral_part = '/dev/fake-part1'
+        swap_part = '/dev/fake-part2'
+        root_part = '/dev/fake-part3'
+        ephemeral_mb = 256
+        ephemeral_format = 'exttest'
+
+        self.mock_mp.return_value = {'ephemeral': ephemeral_part,
+                                     'swap': swap_part,
+                                     'root': root_part}
+        self.mock_ibd.return_value = True
+        calls = [mock.call(root_part),
+                 mock.call(swap_part),
+                 mock.call(ephemeral_part)]
+        res = disk_utils.work_on_disk(self.dev, self.root_mb,
+                                      self.swap_mb, ephemeral_mb,
+                                      ephemeral_format,
+                                      None, self.node_uuid)
+        self.assertEqual(self.mock_ibd.call_args_list, calls)
+        self.mock_mp.assert_called_once_with(self.dev, self.root_mb,
+                                             self.swap_mb, ephemeral_mb,
+                                             self.configdrive_mb,
+                                             self.node_uuid, commit=True,
+                                             boot_option="netboot",
+                                             boot_mode="bios",
+                                             disk_label=None,
+                                             cpu_arch="")
+        self.assertEqual(root_part, res['partitions']['root'])
+        self.assertEqual('uuid', res['root uuid'])
+        self.assertFalse(mock_populate.called)
+
+    @mock.patch.object(utils, 'mkfs', lambda fs, path, label=None: None)
+    @mock.patch.object(disk_utils, 'block_uuid', lambda p: 'uuid')
+    @mock.patch.object(disk_utils, 'populate_image', lambda image_path,
+                       root_path, conv_flags=None: None)
     def test_gpt_disk_label(self):
         ephemeral_part = '/dev/fake-part1'
         swap_part = '/dev/fake-part2'
@@ -216,7 +255,7 @@ class WorkOnDiskTestCase(base.IronicLibTestCase):
         disk_utils.work_on_disk(self.dev, self.root_mb,
                                 self.swap_mb, ephemeral_mb, ephemeral_format,
                                 self.image_path, self.node_uuid,
-                                disk_label='gpt')
+                                disk_label='gpt', conv_flags=None)
         self.assertEqual(self.mock_ibd.call_args_list, calls)
         self.mock_mp.assert_called_once_with(self.dev, self.root_mb,
                                              self.swap_mb, ephemeral_mb,
@@ -224,7 +263,8 @@ class WorkOnDiskTestCase(base.IronicLibTestCase):
                                              self.node_uuid, commit=True,
                                              boot_option="netboot",
                                              boot_mode="bios",
-                                             disk_label='gpt')
+                                             disk_label='gpt',
+                                             cpu_arch="")
 
     @mock.patch.object(disk_utils, 'block_uuid', autospec=True)
     @mock.patch.object(disk_utils, 'populate_image', autospec=True)
@@ -252,12 +292,13 @@ class WorkOnDiskTestCase(base.IronicLibTestCase):
                                              self.node_uuid, commit=True,
                                              boot_option="local",
                                              boot_mode="uefi",
-                                             disk_label=None)
+                                             disk_label=None,
+                                             cpu_arch="")
         self.assertEqual(self.mock_ibd.call_args_list, mock_ibd_calls)
         mock_mkfs.assert_called_once_with(fs='vfat', path=efi_part,
                                           label='efi-part')
         mock_populate_image.assert_called_once_with(self.image_path,
-                                                    root_part)
+                                                    root_part, conv_flags=None)
         mock_block_uuid.assert_any_call(root_part)
         mock_block_uuid.assert_any_call(efi_part)
 
@@ -288,9 +329,64 @@ class WorkOnDiskTestCase(base.IronicLibTestCase):
                                              self.node_uuid, commit=False,
                                              boot_option="netboot",
                                              boot_mode="bios",
-                                             disk_label=None)
+                                             disk_label=None,
+                                             cpu_arch="")
         self.assertFalse(mock_mkfs.called)
 
+    @mock.patch.object(disk_utils, 'block_uuid', autospec=True)
+    @mock.patch.object(disk_utils, 'populate_image', autospec=True)
+    @mock.patch.object(utils, 'mkfs', autospec=True)
+    def test_ppc64le_prep_part(self, mock_mkfs, mock_populate_image,
+                               mock_block_uuid):
+        """Test that PReP partition uuid is returned."""
+        prep_part = '/dev/fake-part1'
+        root_part = '/dev/fake-part2'
+
+        self.mock_mp.return_value = {'PReP Boot partition': prep_part,
+                                     'root': root_part}
+        self.mock_ibd.return_vaue = True
+        calls = [mock.call(root_part),
+                 mock.call(prep_part)]
+        disk_utils.work_on_disk(self.dev, self.root_mb,
+                                self.swap_mb, self.ephemeral_mb,
+                                self.ephemeral_format, self.image_path,
+                                self.node_uuid, boot_option="local",
+                                cpu_arch='ppc64le')
+        self.assertEqual(self.mock_ibd.call_args_list, calls)
+        self.mock_mp.assert_called_once_with(self.dev, self.root_mb,
+                                             self.swap_mb, self.ephemeral_mb,
+                                             self.configdrive_mb,
+                                             self.node_uuid, commit=True,
+                                             boot_option="local",
+                                             boot_mode="bios",
+                                             disk_label=None,
+                                             cpu_arch="ppc64le")
+        self.assertFalse(mock_mkfs.called)
+
+    @mock.patch.object(disk_utils, 'block_uuid', autospec=True)
+    @mock.patch.object(disk_utils, 'populate_image', autospec=True)
+    @mock.patch.object(utils, 'mkfs', autospec=True)
+    def test_convert_to_sparse(self, mock_mkfs, mock_populate_image,
+                               mock_block_uuid):
+        ephemeral_part = '/dev/fake-part1'
+        swap_part = '/dev/fake-part2'
+        root_part = '/dev/fake-part3'
+        ephemeral_mb = 256
+        ephemeral_format = 'exttest'
+
+        self.mock_mp.return_value = {'ephemeral': ephemeral_part,
+                                     'swap': swap_part,
+                                     'root': root_part}
+        self.mock_ibd.return_value = True
+        disk_utils.work_on_disk(self.dev, self.root_mb,
+                                self.swap_mb, ephemeral_mb, ephemeral_format,
+                                self.image_path, self.node_uuid,
+                                disk_label='gpt', conv_flags='sparse')
+
+        mock_populate_image.assert_called_once_with(self.image_path,
+                                                    root_part,
+                                                    conv_flags='sparse')
+
 
 @mock.patch.object(utils, 'execute', autospec=True)
 class MakePartitionsTestCase(base.IronicLibTestCase):
@@ -314,17 +410,16 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                 '--', 'unit', 'MiB', 'mklabel', label]
 
     def _test_make_partitions(self, mock_exc, boot_option, boot_mode='bios',
-                              disk_label=None):
+                              disk_label=None, cpu_arch=""):
         mock_exc.return_value = ('', '')
         disk_utils.make_partitions(self.dev, self.root_mb, self.swap_mb,
                                    self.ephemeral_mb, self.configdrive_mb,
                                    self.node_uuid, boot_option=boot_option,
-                                   boot_mode=boot_mode, disk_label=disk_label)
-
-        _s = lambda x, sz: x + sz
+                                   boot_mode=boot_mode, disk_label=disk_label,
+                                   cpu_arch=cpu_arch)
 
         if boot_option == "local" and boot_mode == "uefi":
-            add_efi_sz = lambda x: str(_s(x, self.efi_size))
+            add_efi_sz = lambda x: str(x + self.efi_size)
             expected_mkpart = ['mkpart', 'primary', 'fat32', '1',
                                add_efi_sz(1),
                                'set', '1', 'boot', 'on',
@@ -335,14 +430,28 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
         else:
             if boot_option == "local":
                 if disk_label == "gpt":
-                    add_bios_sz = lambda x: str(_s(x, self.bios_size))
-                    expected_mkpart = ['mkpart', 'primary', '', '1',
-                                       add_bios_sz(1),
-                                       'set', '1', 'bios_grub', 'on',
+                    if cpu_arch.startswith('ppc64'):
+                        expected_mkpart = ['mkpart', 'primary', '', '1', '9',
+                                           'set', '1', 'prep', 'on',
+                                           'mkpart', 'primary', 'linux-swap',
+                                           '9', '521', 'mkpart', 'primary',
+                                           '', '521', '1545']
+                    else:
+                        add_bios_sz = lambda x: str(x + self.bios_size)
+                        expected_mkpart = ['mkpart', 'primary', '', '1',
+                                           add_bios_sz(1),
+                                           'set', '1', 'bios_grub', 'on',
+                                           'mkpart', 'primary', 'linux-swap',
+                                           add_bios_sz(1), add_bios_sz(513),
+                                           'mkpart', 'primary', '',
+                                           add_bios_sz(513), add_bios_sz(1537)]
+                elif cpu_arch.startswith('ppc64'):
+                    expected_mkpart = ['mkpart', 'primary', '', '1', '9',
+                                       'set', '1', 'boot', 'on',
+                                       'set', '1', 'prep', 'on',
                                        'mkpart', 'primary', 'linux-swap',
-                                       add_bios_sz(1), add_bios_sz(513),
-                                       'mkpart', 'primary', '',
-                                       add_bios_sz(513), add_bios_sz(1537)]
+                                       '9', '521', 'mkpart', 'primary',
+                                       '', '521', '1545']
                 else:
                     expected_mkpart = ['mkpart', 'primary', 'linux-swap', '1',
                                        '513', 'mkpart', 'primary', '', '513',
@@ -379,6 +488,14 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
         self._test_make_partitions(mock_exc, boot_option="netboot",
                                    disk_label="gpt")
 
+    def test_make_partitions_mbr_with_prep(self, mock_exc):
+        self._test_make_partitions(mock_exc, boot_option="local",
+                                   disk_label="msdos", cpu_arch="ppc64le")
+
+    def test_make_partitions_gpt_with_prep(self, mock_exc):
+        self._test_make_partitions(mock_exc, boot_option="local",
+                                   disk_label="gpt", cpu_arch="ppc64le")
+
     def test_make_partitions_with_ephemeral(self, mock_exc):
         self.ephemeral_mb = 2048
         expected_mkpart = ['mkpart', 'primary', '', '1', '2049',
@@ -554,7 +671,15 @@ class PopulateImageTestCase(base.IronicLibTestCase):
         type(mock_qinfo.return_value).file_format = mock.PropertyMock(
             return_value='raw')
         disk_utils.populate_image('src', 'dst')
-        mock_dd.assert_called_once_with('src', 'dst')
+        mock_dd.assert_called_once_with('src', 'dst', conv_flags=None)
+        self.assertFalse(mock_cg.called)
+
+    def test_populate_raw_image_with_convert(self, mock_cg, mock_qinfo,
+                                             mock_dd):
+        type(mock_qinfo.return_value).file_format = mock.PropertyMock(
+            return_value='raw')
+        disk_utils.populate_image('src', 'dst', conv_flags='sparse')
+        mock_dd.assert_called_once_with('src', 'dst', conv_flags='sparse')
         self.assertFalse(mock_cg.called)
 
     def test_populate_qcow2_image(self, mock_cg, mock_qinfo, mock_dd):
@@ -809,6 +934,22 @@ class OtherFunctionTestCase(base.IronicLibTestCase):
                                              '/dev/fake', run_as_root=True,
                                              use_standard_locale=True)
 
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_block_uuid_fallback_to_uuid(self, mock_execute):
+        mock_execute.side_effect = [('', ''),
+                                    ('value', '')]
+        self.assertEqual('value',
+                         disk_utils.block_uuid('/dev/fake'))
+        execute_calls = [
+            mock.call('blkid', '-s', 'UUID', '-o', 'value',
+                      '/dev/fake', check_exit_code=[0],
+                      run_as_root=True),
+            mock.call('blkid', '-s', 'PARTUUID', '-o', 'value',
+                      '/dev/fake', check_exit_code=[0],
+                      run_as_root=True)
+        ]
+        mock_execute.assert_has_calls(execute_calls)
+
 
 @mock.patch.object(utils, 'execute', autospec=True)
 class WholeDiskPartitionTestCases(base.IronicLibTestCase):
@@ -828,7 +969,23 @@ class WholeDiskPartitionTestCases(base.IronicLibTestCase):
                                                     self.node_uuid)
         self.assertEqual(part_result, result)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True),
+            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
+            mock.call('lsblk', '-Po', 'name,label', self.dev,
+                      check_exit_code=[0, 1],
+                      use_standard_locale=True, run_as_root=True)
+        ]
+        mock_execute.assert_has_calls(execute_calls)
+
+    def test_get_partition_present_uppercase(self, mock_execute):
+        lsblk_output = 'NAME="fake12" LABEL="CONFIG-2"\n'
+        part_result = '/dev/fake12'
+        mock_execute.side_effect = [(None, ''), (lsblk_output, '')]
+        result = disk_utils._get_labelled_partition(self.dev,
+                                                    self.config_part_label,
+                                                    self.node_uuid)
+        self.assertEqual(part_result, result)
+        execute_calls = [
+            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
                       use_standard_locale=True, run_as_root=True)
@@ -843,7 +1000,7 @@ class WholeDiskPartitionTestCases(base.IronicLibTestCase):
                                                     self.node_uuid)
         self.assertIsNone(result)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True),
+            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
                       use_standard_locale=True, run_as_root=True)
@@ -861,7 +1018,7 @@ class WholeDiskPartitionTestCases(base.IronicLibTestCase):
                                disk_utils._get_labelled_partition, self.dev,
                                self.config_part_label, self.node_uuid)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True),
+            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
                       use_standard_locale=True, run_as_root=True)
@@ -875,8 +1032,8 @@ class WholeDiskPartitionTestCases(base.IronicLibTestCase):
                                'Failed to retrieve partition labels',
                                disk_utils._get_labelled_partition, self.dev,
                                self.config_part_label, self.node_uuid)
-        mock_execute.assert_called_once_with('partprobe', self.dev,
-                                             run_as_root=True)
+        mock_execute.assert_called_once_with(
+            'partprobe', self.dev, run_as_root=True, attempts=10)
         self.assertEqual(1, mock_log.call_count)
 
     def _test_is_disk_larger_than_max_size(self, mock_execute, blk_out):
@@ -948,18 +1105,17 @@ class WholeDiskPartitionTestCases(base.IronicLibTestCase):
         self.assertEqual(1, mock_log.call_count)
 
     def test_fix_gpt_structs_fix_required(self, mock_execute):
-        partprobe_err = """
-Error: The backup GPT table is not at the end of the disk, as it should be.
-This might mean that another operating system believes the disk is smaller.
-Fix, by moving the backup to the end (and removing the old backup)?
-Warning: Not all of the space available to /dev/sdb appears to be used,
-you can fix the GPT to use all of the space (an extra 581456476 blocks)
-or continue with the current setting?
+        sgdisk_v_output = """
+Problem: The secondary header's self-pointer indicates that it doesn't reside
+at the end of the disk. If you've added a disk to a RAID array, use the 'e'
+option on the experts' menu to adjust the secondary header's and partition
+table's locations.
+
+Identified 1 problems!
 """
-        mock_execute.return_value = ('', partprobe_err)
+        mock_execute.return_value = (sgdisk_v_output, '')
         execute_calls = [
-            mock.call('partprobe', '/dev/fake', use_standard_locale=True,
-                      run_as_root=True),
+            mock.call('sgdisk', '-v', '/dev/fake', run_as_root=True),
             mock.call('sgdisk', '-e', '/dev/fake', run_as_root=True)
         ]
         disk_utils._fix_gpt_structs('/dev/fake', self.node_uuid)
@@ -969,8 +1125,7 @@ or continue with the current setting?
         mock_execute.return_value = ('', '')
 
         disk_utils._fix_gpt_structs('/dev/fake', self.node_uuid)
-        mock_execute.assert_called_once_with('partprobe', '/dev/fake',
-                                             use_standard_locale=True,
+        mock_execute.assert_called_once_with('sgdisk', '-v', '/dev/fake',
                                              run_as_root=True)
 
     @mock.patch.object(disk_utils.LOG, 'error', autospec=True)
@@ -980,8 +1135,7 @@ or continue with the current setting?
                                'Failed to fix GPT data structures on disk',
                                disk_utils._fix_gpt_structs,
                                self.dev, self.node_uuid)
-        mock_execute.assert_called_once_with('partprobe', '/dev/fake',
-                                             use_standard_locale=True,
+        mock_execute.assert_called_once_with('sgdisk', '-v', '/dev/fake',
                                              run_as_root=True)
         self.assertEqual(1, mock_log.call_count)
 
@@ -999,6 +1153,8 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                        autospec=True)
     @mock.patch.object(disk_utils, 'dd',
                        autospec=True)
+    @mock.patch.object(disk_utils, 'fix_gpt_partition',
+                       autospec=True)
     @mock.patch.object(disk_utils, '_fix_gpt_structs',
                        autospec=True)
     @mock.patch.object(disk_utils, '_is_disk_gpt_partitioned',
@@ -1011,8 +1167,8 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                        autospec=True)
     def test_create_partition_exists(self, mock_get_configdrive,
                                      mock_get_labelled_partition,
-                                     mock_list_partitions,
-                                     mock_is_disk_gpt, mock_fix_gpt,
+                                     mock_list_partitions, mock_is_disk_gpt,
+                                     mock_fix_gpt, mock_fix_gpt_partition,
                                      mock_dd, mock_unlink, mock_execute):
         config_url = 'http://1.2.3.4/cd'
         configdrive_part = '/dev/fake-part1'
@@ -1023,13 +1179,15 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
         mock_get_configdrive.return_value = (configdrive_mb, configdrive_file)
         disk_utils.create_config_drive_partition(self.node_uuid, self.dev,
                                                  config_url)
+        mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
         mock_get_labelled_partition.assert_called_with(self.dev,
                                                        self.config_part_label,
                                                        self.node_uuid)
         self.assertFalse(mock_list_partitions.called)
-        self.assertFalse(mock_is_disk_gpt.called)
         self.assertFalse(mock_execute.called)
+        self.assertFalse(mock_is_disk_gpt.called)
+        self.assertFalse(mock_fix_gpt.called)
         mock_dd.assert_called_with(configdrive_file, configdrive_part)
         mock_unlink.assert_called_with(configdrive_file)
 
@@ -1038,6 +1196,8 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                        autospec=True)
     @mock.patch.object(disk_utils, 'dd',
                        autospec=True)
+    @mock.patch.object(disk_utils, 'fix_gpt_partition',
+                       autospec=True)
     @mock.patch.object(disk_utils, '_fix_gpt_structs',
                        autospec=True)
     @mock.patch.object(disk_utils, '_is_disk_gpt_partitioned',
@@ -1051,8 +1211,8 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
     def test_create_partition_gpt(self, mock_get_configdrive,
                                   mock_get_labelled_partition,
                                   mock_list_partitions, mock_is_disk_gpt,
-                                  mock_fix_gpt, mock_dd, mock_unlink,
-                                  mock_execute):
+                                  mock_fix_gpt, mock_fix_gpt_partition,
+                                  mock_dd, mock_unlink, mock_execute):
         config_url = 'http://1.2.3.4/cd'
         configdrive_file = '/tmp/xyz'
         configdrive_mb = 10
@@ -1086,6 +1246,11 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
         mock_execute.assert_has_calls([
             mock.call('sgdisk', '-n', '0:-64MB:0', self.dev,
                       run_as_root=True),
+            mock.call('sync'),
+            mock.call('udevadm', 'settle'),
+            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
+            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
+
             mock.call('udevadm', 'settle'),
             mock.call('test', '-e', expected_part, attempts=15,
                       check_exit_code=[0], delay_on_retry=True)
@@ -1093,7 +1258,9 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
 
         self.assertEqual(2, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
-        mock_fix_gpt.assert_called_with(self.dev, self.node_uuid)
+        mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
+        self.assertFalse(mock_fix_gpt.called)
+        mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
         mock_dd.assert_called_with(configdrive_file, expected_part)
         mock_unlink.assert_called_with(configdrive_file)
 
@@ -1106,6 +1273,8 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                        autospec=True)
     @mock.patch.object(disk_utils, '_is_disk_larger_than_max_size',
                        autospec=True)
+    @mock.patch.object(disk_utils, 'fix_gpt_partition',
+                       autospec=True)
     @mock.patch.object(disk_utils, '_fix_gpt_structs',
                        autospec=True)
     @mock.patch.object(disk_utils, '_is_disk_gpt_partitioned',
@@ -1120,6 +1289,7 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                    mock_get_labelled_partition,
                                    mock_list_partitions,
                                    mock_is_disk_gpt, mock_fix_gpt,
+                                   mock_fix_gpt_partition,
                                    mock_disk_exceeds, mock_dd,
                                    mock_unlink, mock_log, mock_execute,
                                    mock_count, disk_size_exceeds_max=False,
@@ -1182,16 +1352,22 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                     '-0', run_as_root=True)
         mock_execute.assert_has_calls([
             parted_call,
+            mock.call('sync'),
+            mock.call('udevadm', 'settle'),
+            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
+            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
             mock.call('udevadm', 'settle'),
             mock.call('test', '-e', expected_part, attempts=15,
                       check_exit_code=[0], delay_on_retry=True)
         ])
         self.assertEqual(2, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
+        mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
         mock_disk_exceeds.assert_called_with(self.dev, self.node_uuid)
         mock_dd.assert_called_with(configdrive_file, expected_part)
         mock_unlink.assert_called_with(configdrive_file)
         self.assertFalse(mock_fix_gpt.called)
+        self.assertFalse(mock_fix_gpt.called)
         mock_count.assert_called_with(self.dev)
 
     def test__create_partition_mbr_disk_under_2TB(self):
@@ -1222,6 +1398,8 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                        autospec=True)
     @mock.patch.object(disk_utils, '_is_disk_larger_than_max_size',
                        autospec=True)
+    @mock.patch.object(disk_utils, 'fix_gpt_partition',
+                       autospec=True)
     @mock.patch.object(disk_utils, '_fix_gpt_structs',
                        autospec=True)
     @mock.patch.object(disk_utils, '_is_disk_gpt_partitioned',
@@ -1236,6 +1414,7 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                                mock_get_labelled_partition,
                                                mock_list_partitions,
                                                mock_is_disk_gpt, mock_fix_gpt,
+                                               mock_fix_gpt_partition,
                                                mock_disk_exceeds, mock_dd,
                                                mock_unlink, mock_execute,
                                                mock_count):
@@ -1273,11 +1452,19 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                self.node_uuid, self.dev, config_url)
 
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
-        mock_execute.assert_called_with('parted', '-a', 'optimal', '-s', '--',
-                                        self.dev, 'mkpart', 'primary',
-                                        'fat32', '-64MiB', '-0',
-                                        run_as_root=True)
+        mock_execute.assert_has_calls([
+            mock.call('parted', '-a', 'optimal', '-s', '--',
+                      self.dev, 'mkpart', 'primary',
+                      'fat32', '-64MiB', '-0',
+                      run_as_root=True),
+            mock.call('sync'),
+            mock.call('udevadm', 'settle'),
+            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
+            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
+        ])
+
         self.assertEqual(2, mock_list_partitions.call_count)
+        mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         mock_disk_exceeds.assert_called_with(self.dev, self.node_uuid)
         self.assertFalse(mock_fix_gpt.called)
@@ -1293,6 +1480,8 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                        autospec=True)
     @mock.patch.object(disk_utils, '_is_disk_larger_than_max_size',
                        autospec=True)
+    @mock.patch.object(disk_utils, 'fix_gpt_partition',
+                       autospec=True)
     @mock.patch.object(disk_utils, '_fix_gpt_structs',
                        autospec=True)
     @mock.patch.object(disk_utils, '_is_disk_gpt_partitioned',
@@ -1307,6 +1496,7 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                               mock_get_labelled_partition,
                                               mock_list_partitions,
                                               mock_is_disk_gpt, mock_fix_gpt,
+                                              mock_fix_gpt_partition,
                                               mock_disk_exceeds, mock_dd,
                                               mock_unlink, mock_execute,
                                               mock_count):
@@ -1343,6 +1533,7 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                         'fat32', '-64MiB', '-0',
                                         run_as_root=True)
         self.assertEqual(1, mock_list_partitions.call_count)
+        mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         mock_disk_exceeds.assert_called_with(self.dev, self.node_uuid)
         self.assertFalse(mock_fix_gpt.called)
@@ -1355,6 +1546,8 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                        autospec=True)
     @mock.patch.object(disk_utils, 'dd',
                        autospec=True)
+    @mock.patch.object(disk_utils, 'fix_gpt_partition',
+                       autospec=True)
     @mock.patch.object(disk_utils, '_fix_gpt_structs',
                        autospec=True)
     @mock.patch.object(disk_utils, '_is_disk_gpt_partitioned',
@@ -1369,6 +1562,7 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                                mock_get_labelled_partition,
                                                mock_list_partitions,
                                                mock_is_disk_gpt, mock_fix_gpt,
+                                               mock_fix_gpt_partition,
                                                mock_dd, mock_unlink,
                                                mock_count):
         config_url = 'http://1.2.3.4/cd'
@@ -1398,6 +1592,7 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
 
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
         self.assertEqual(1, mock_list_partitions.call_count)
+        mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         self.assertFalse(mock_fix_gpt.called)
         self.assertFalse(mock_dd.called)
@@ -1433,6 +1628,8 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
     @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(utils, 'unlink_without_raise',
                        autospec=True)
+    @mock.patch.object(disk_utils, 'fix_gpt_partition',
+                       autospec=True)
     @mock.patch.object(disk_utils, '_is_disk_gpt_partitioned',
                        autospec=True)
     @mock.patch.object(disk_utils, '_get_labelled_partition',
@@ -1441,7 +1638,8 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                        autospec=True)
     def test_create_partition_conf_drive_error_counting(
             self, mock_get_configdrive, mock_get_labelled_partition,
-            mock_is_disk_gpt, mock_unlink, mock_execute, mock_count):
+            mock_is_disk_gpt, mock_fix_gpt_partition,
+            mock_unlink, mock_execute, mock_count):
         config_url = 'http://1.2.3.4/cd'
         configdrive_file = '/tmp/xyz'
         configdrive_mb = 10
@@ -1458,5 +1656,6 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
 
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
         mock_unlink.assert_called_with(configdrive_file)
+        mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         mock_count.assert_called_once_with(self.dev)
